@@ -32,9 +32,10 @@ has synchronous  => ( is => 'ro', isa => Enum [qw[FULL NORMAL OFF]],
 has cache_size   => ( is => 'ro', isa => Int,  default => -1_048_576 );                                              # 0+ - pages,  -kilobytes, default 1G
 has foreign_keys => ( is => 'ro', isa => Bool, default => 1 );
 
-has is_sqlite    => ( is => 'ro', isa => Bool,    default  => 1, init_arg => undef );
-has h            => ( is => 'ro', isa => Object,  init_arg => undef );
-has prepared_sth => ( is => 'ro', isa => HashRef, init_arg => undef );
+has is_sqlite    => ( is => 'ro', isa => Bool,      default  => 1, init_arg => undef );
+has h            => ( is => 'ro', isa => Object,    init_arg => undef );
+has prepared_sth => ( is => 'ro', isa => HashRef,   init_arg => undef );
+has query        => ( is => 'ro', isa => ScalarRef, init_arg => undef );                                             # ref to the last query
 
 # SQLite types
 const our $SQLITE_UNKNOWN => 0;
@@ -252,6 +253,8 @@ sub _exec_sth ( $self, $query, @args ) {
         $sth = $self->{prepared_sth}->{ $query->{id} };
 
         if ( !defined $sth ) {
+            $self->{query} = \$query->{query};
+
             $sth = $dbh->prepare( $query->{query} );
 
             return $rows, $sth, \%args, $cb if defined $DBI::err;
@@ -271,6 +274,8 @@ sub _exec_sth ( $self, $query, @args ) {
 
     # prepare sth
     if ( !defined $sth ) {
+        $self->{query} = \$query;
+
         $sth = $dbh->prepare($query);
 
         return $rows, $sth, \%args, $cb if defined $DBI::err;
@@ -306,6 +311,16 @@ sub _exec_sth ( $self, $query, @args ) {
     }
 }
 
+sub _warn ($self) {
+    warn qq[DBI: "$DBI::errstr"] . ( defined $self->{query} ? qq[, current query: "$self->{query}->$*"] : q[] );
+
+    return;
+}
+
+sub _die ($self) {
+    die qq[DBI: "$DBI::errstr"] . ( defined $self->{query} ? qq[, current query: "$self->{query}->$*"] : q[] );
+}
+
 # PUBLIC DBI METHODS
 sub do ( $self, $query, @args ) {    ## no critic qw[Subroutines::ProhibitBuiltinHomonyms]
 
@@ -324,12 +339,14 @@ sub do ( $self, $query, @args ) {    ## no critic qw[Subroutines::ProhibitBuilti
             $sth = $self->{prepared_sth}->{ $query->{id} };
 
             if ( !defined $sth ) {
+                $self->{query} = \$query->{query};
+
                 $sth = $dbh->prepare( $query->{query} );
 
                 # check error
                 if ( defined $DBI::err ) {
-                    if ( defined $cb ) { warn "DBI: $DBI::errstr"; $cb->( $self, result( [ 500, $DBI::errstr ], rows => $rows ), undef ); return $rows }
-                    else               { die $DBI::errstr }
+                    if ( defined $cb ) { $self->_warn; $cb->( $self, result( [ 500, $DBI::errstr ], rows => $rows ), undef ); return $rows }
+                    else               { $self->_die }
                 }
 
                 $self->{prepared_sth}->{ $query->{id} } = $sth;
@@ -343,7 +360,7 @@ sub do ( $self, $query, @args ) {    ## no critic qw[Subroutines::ProhibitBuilti
             $sth = $query;
         }
         else {
-            if ( defined $cb ) { warn "DBI: $DBI::errstr"; $cb->( $self, result( [ 500, 'Invalid STH class' ], rows => $rows ), undef ); return $rows }
+            if ( defined $cb ) { $self->_warn; $cb->( $self, result( [ 500, 'Invalid STH class' ], rows => $rows ), undef ); return $rows }
             else               { die 'Invalid STH class' }
         }
 
@@ -370,8 +387,8 @@ sub do ( $self, $query, @args ) {    ## no critic qw[Subroutines::ProhibitBuilti
 
         # check error
         if ( defined $DBI::err ) {
-            if ( defined $cb ) { warn "DBI: $DBI::errstr"; $cb->( $self, result( [ 500, $DBI::errstr ], rows => $rows ), undef ); return $rows }
-            else               { die $DBI::errstr }
+            if ( defined $cb ) { $self->_warn; $cb->( $self, result( [ 500, $DBI::errstr ], rows => $rows ), undef ); return $rows }
+            else               { $self->_die }
         }
 
         $rows = 0 if $rows == 0;    # convert "0E0" to "0"
@@ -390,12 +407,14 @@ sub do ( $self, $query, @args ) {    ## no critic qw[Subroutines::ProhibitBuilti
     # execute query directly without prepare and bind params
     # multiple queries in single statement are allowed
     if ( !defined $bind ) {
+        $self->{query} = \$query;
+
         $rows = DBD::SQLite::db::_do( $dbh, $query );
 
         # check error
         if ( defined $DBI::err ) {
-            if ( defined $cb ) { warn "DBI: $DBI::errstr"; $cb->( $self, result( [ 500, $DBI::errstr ], rows => $rows ), undef ); return $rows }
-            else               { die $DBI::errstr }
+            if ( defined $cb ) { $self->_warn; $cb->( $self, result( [ 500, $DBI::errstr ], rows => $rows ), undef ); return $rows }
+            else               { $self->_die }
         }
 
         $rows = 0 if $rows == 0;    # convert "0E0" to "0"
@@ -411,14 +430,15 @@ sub do ( $self, $query, @args ) {    ## no critic qw[Subroutines::ProhibitBuilti
         my $bind_pos = 0;
 
         while ($query) {
+            $self->{query} = \$query;
 
             # prepare sth
             my $sth = $dbh->prepare($query);
 
             # prepare sth error
             if ( defined $DBI::err ) {
-                if ( defined $cb ) { warn "DBI: $DBI::errstr"; $cb->( $self, result( [ 500, $DBI::errstr ], rows => $rows ), undef ); return $rows }
-                else               { die $DBI::errstr }
+                if ( defined $cb ) { $self->_warn; $cb->( $self, result( [ 500, $DBI::errstr ], rows => $rows ), undef ); return $rows }
+                else               { $self->_die }
             }
 
             # make a copy
@@ -439,8 +459,8 @@ sub do ( $self, $query, @args ) {    ## no critic qw[Subroutines::ProhibitBuilti
 
             # execute error
             if ( defined $DBI::err ) {
-                if ( defined $cb ) { warn "DBI: $DBI::errstr"; $cb->( $self, result( [ 500, $DBI::errstr ], rows => $rows ), undef ); return $rows }
-                else               { die $DBI::errstr }
+                if ( defined $cb ) { $self->_warn; $cb->( $self, result( [ 500, $DBI::errstr ], rows => $rows ), undef ); return $rows }
+                else               { $self->_die }
             }
 
             $rows += $sth->rows;
@@ -460,8 +480,8 @@ sub selectall ( $self, @ ) {
 
     # check error
     if ( defined $DBI::err ) {
-        if ( defined $cb ) { warn "DBI: $DBI::errstr"; $cb->( $self, result( [ 500, $DBI::errstr ], rows => $rows ), undef ); return }
-        else               { die $DBI::errstr }
+        if ( defined $cb ) { $self->_warn; $cb->( $self, result( [ 500, $DBI::errstr ], rows => $rows ), undef ); return }
+        else               { $self->_die }
     }
 
     my $data;
@@ -475,8 +495,8 @@ sub selectall ( $self, @ ) {
 
         # check error
         if ( defined $DBI::err ) {
-            if ( defined $cb ) { warn "DBI: $DBI::errstr"; $cb->( $self, result( [ 500, $DBI::errstr ], rows => $rows ), undef ); return }
-            else               { die $DBI::errstr }
+            if ( defined $cb ) { $self->_warn; $cb->( $self, result( [ 500, $DBI::errstr ], rows => $rows ), undef ); return }
+            else               { $self->_die }
         }
 
         undef $data if !$data->%*;
@@ -497,8 +517,8 @@ sub selectall_arrayref ( $self, @ ) {
 
     # check error
     if ( defined $DBI::err ) {
-        if ( defined $cb ) { warn "DBI: $DBI::errstr"; $cb->( $self, result( [ 500, $DBI::errstr ], rows => $rows ), undef ); return }
-        else               { die $DBI::errstr }
+        if ( defined $cb ) { $self->_warn; $cb->( $self, result( [ 500, $DBI::errstr ], rows => $rows ), undef ); return }
+        else               { $self->_die }
     }
 
     my $data = $sth->fetchall_arrayref( undef, undef );
@@ -515,8 +535,8 @@ sub selectrow ( $self, @ ) {
 
     # check error
     if ( defined $DBI::err ) {
-        if ( defined $cb ) { warn "DBI: $DBI::errstr"; $cb->( $self, result( [ 500, $DBI::errstr ], rows => $rows ), undef ); return }
-        else               { die $DBI::errstr }
+        if ( defined $cb ) { $self->_warn; $cb->( $self, result( [ 500, $DBI::errstr ], rows => $rows ), undef ); return }
+        else               { $self->_die }
     }
 
     my $data = $sth->fetchrow_hashref;
@@ -533,8 +553,8 @@ sub selectrow_arrayref ( $self, @ ) {
 
     # check error
     if ( defined $DBI::err ) {
-        if ( defined $cb ) { warn "DBI: $DBI::errstr"; $cb->( $self, result( [ 500, $DBI::errstr ], rows => $rows ), undef ); return }
-        else               { die $DBI::errstr }
+        if ( defined $cb ) { $self->_warn; $cb->( $self, result( [ 500, $DBI::errstr ], rows => $rows ), undef ); return }
+        else               { $self->_die }
     }
 
     my $data = $sth->fetchrow_arrayref;
@@ -552,8 +572,8 @@ sub selectcol ( $self, @ ) {
 
     # check error
     if ( defined $DBI::err ) {
-        if ( defined $cb ) { warn "DBI: $DBI::errstr"; $cb->( $self, result( [ 500, $DBI::errstr ], rows => $rows ), undef ); return }
-        else               { die $DBI::errstr }
+        if ( defined $cb ) { $self->_warn; $cb->( $self, result( [ 500, $DBI::errstr ], rows => $rows ), undef ); return }
+        else               { $self->_die }
     }
 
     my ( $data, $name2idx, $idx, @vals );
@@ -565,7 +585,7 @@ sub selectcol ( $self, @ ) {
     for my $col ( is_plain_arrayref $args->{col} ? $args->{col}->@* : $args->{col} ) {
         if ( looks_like_number $col) {
             if ( $col > $num_of_fields ) {
-                if ( defined $cb ) { warn "DBI: $DBI::errstr"; $cb->( $self, result( [ 500, qq[Invalid column index: "$col"] ], rows => $rows ), undef ); return }
+                if ( defined $cb ) { $self->_warn; $cb->( $self, result( [ 500, qq[Invalid column index: "$col"] ], rows => $rows ), undef ); return }
                 else               { die qq[Invalid column index: "$col"] }
             }
 
@@ -575,7 +595,7 @@ sub selectcol ( $self, @ ) {
             $name2idx //= $sth->{NAME_hash};
 
             if ( !exists $name2idx->{$col} ) {
-                if ( defined $cb ) { warn "DBI: $DBI::errstr"; $cb->( $self, result( [ 500, qq[Invalid column name: "$col"] ], rows => $rows ), undef ); return }
+                if ( defined $cb ) { $self->_warn; $cb->( $self, result( [ 500, qq[Invalid column name: "$col"] ], rows => $rows ), undef ); return }
                 else               { die qq[Invalid column name: "$col"] }
             }
 
@@ -596,8 +616,8 @@ sub begin_work ( $self, $cb = undef ) {
 
     # check error
     if ( defined $DBI::err ) {
-        if ( defined $cb ) { warn "DBI: $DBI::errstr"; $cb->( $self, result [ 500, $DBI::errstr ] ); return }
-        else               { die $DBI::errstr }
+        if ( defined $cb ) { $self->_warn; $cb->( $self, result [ 500, $DBI::errstr ] ); return }
+        else               { $self->_die }
     }
 
     $cb->( $self, result 200 ) if defined $cb;
@@ -610,8 +630,8 @@ sub commit ( $self, $cb = undef ) {
 
     # check error
     if ( defined $DBI::err ) {
-        if ( defined $cb ) { warn "DBI: $DBI::errstr"; $cb->( $self, result [ 500, $DBI::errstr ] ); return }
-        else               { die $DBI::errstr }
+        if ( defined $cb ) { $self->_warn; $cb->( $self, result [ 500, $DBI::errstr ] ); return }
+        else               { $self->_die }
     }
 
     $cb->( $self, result 200 ) if defined $cb;
@@ -624,8 +644,8 @@ sub rollback ( $self, $cb = undef ) {
 
     # check error
     if ( defined $DBI::err ) {
-        if ( defined $cb ) { warn "DBI: $DBI::errstr"; $cb->( $self, result [ 500, $DBI::errstr ] ); return }
-        else               { die $DBI::errstr }
+        if ( defined $cb ) { $self->_warn; $cb->( $self, result [ 500, $DBI::errstr ] ); return }
+        else               { $self->_die }
     }
 
     $cb->( $self, result 200 ) if defined $cb;
@@ -661,18 +681,18 @@ sub attach ( $self, $name, $path = undef ) {
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ## | Sev. | Lines                | Policy                                                                                                         |
 ## |======+======================+================================================================================================================|
-## |    3 | 153                  | Subroutines::ProhibitUnusedPrivateSubroutines - Private subroutine/method '_get_schema_patch_table_query'      |
+## |    3 | 154                  | Subroutines::ProhibitUnusedPrivateSubroutines - Private subroutine/method '_get_schema_patch_table_query'      |
 ## |      |                      | declared but not used                                                                                          |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    3 | 310                  | Subroutines::ProhibitExcessComplexity - Subroutine "do" with high complexity score (40)                        |
+## |    3 | 325                  | Subroutines::ProhibitExcessComplexity - Subroutine "do" with high complexity score (40)                        |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    3 | 393                  | Subroutines::ProtectPrivateSubs - Private subroutine/method used                                               |
+## |    3 | 412                  | Subroutines::ProtectPrivateSubs - Private subroutine/method used                                               |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    2 | 201                  | ValuesAndExpressions::ProhibitEscapedCharacters - Numeric escapes in interpolated string                       |
+## |    2 | 202                  | ValuesAndExpressions::ProhibitEscapedCharacters - Numeric escapes in interpolated string                       |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    2 | 285, 357, 430        | ControlStructures::ProhibitCStyleForLoops - C-style "for" loop used                                            |
+## |    2 | 290, 374, 450        | ControlStructures::ProhibitCStyleForLoops - C-style "for" loop used                                            |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    2 | 586                  | ControlStructures::ProhibitPostfixControls - Postfix control "while" used                                      |
+## |    2 | 606                  | ControlStructures::ProhibitPostfixControls - Postfix control "while" used                                      |
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ##
 ## -----SOURCE FILTER LOG END-----
